// See LICENSE.TXT for license agreement.

#ifndef IMAGE_H
#define IMAGE_H

#include "misc.h" // for Vertex

typedef struct Image_t
{
	int width, height, texWidth, texHeight;
	int origWidth, origHeight; // to restore after scaling
	int colorMode;
	unsigned char* data;
	unsigned char swizzled; // true or false value
	
	Vertex* vertices; // always will be an array of 4 or NULL
	
} Image;

// newImage - create solid color image
//--------------------------------------------------------
Image* newImage(int width, int height, unsigned int color);

// loadImage - load image from a png file
//-----------------------------------------
Image* loadImage(const char fileName[]);

// swizzleImage - swizzles the texture
//----------------------------------------
void swizzleImage(Image* img);

// createImageVerts
//-------------------------------------
void createImageVerts(Image* img);

// freeImage - free image from memory
//---------------------------------------
void freeImage(Image* img);

// drawImage - draws the image to the screen by using the shape
// 			primitive GU_TRIANGLE_STRIP with the texture on it
//------------------------------------------------------------
void drawImage(Image* img, int x, int y, float radians);

// drawImagePart - draws part of the image.
//----------------------------------------------------
void drawImagePart(Image* img, int dx, int dy, int srcX,
					int srcY, int srcW, int srcH, float rads);

// drawImageToImage - don't use this with function. it doesn't
//	work properly yet. I might fix it in the next release but
//	it seems mostly useless.
//
void drawImageToImage(Image* srcImg, Image* dstImg, int dx, int dy,
					int srcX, int srcY, int srcW, int srcH);
					
// setImageZ - sets the z location of the image. The image
//	with the lowest Z is drawn below other images. The Z
//	value MUST be between -1 and 1. It can be a fraction, such as
//	0.31, -0.6, etc., of course.
//----------------------------------------------------------------
void setImageZ(Image* img, float z);

// stretchImage - stretches the image by modifying the coord
//		locations.
//----------------------------------------------------------
void stretchImage(Image* img, float xScale, float yScale);

// restoreImage - restores to original scale after scaling
//--------------------------------------------------------
void restoreImage(Image* img);

u32 getImagePixelAt(Image* img, int x, int y);
void setImagePixelAt(Image* img, int x,int y, u32 color);

// nextPowOf2 - used to find a suitable texture height that is a power
//	of two.
//-------------------------------------------------------------------
int nextPowOf2(int dimension);

#endif
