// (See LICENSE.TXT for license agreement.)

#include <pspkernel.h>
#include <pspdisplay.h>
#include <pspdebug.h>
#include "callbacks.h"

#include "misc.h"
#include "image.h"

#define PI (3.14159) // close enough
#define DEG2RAD(a) (a*PI/180.0f)

PSP_MODULE_INFO("HMP2D_IMGLIB",0,1,1);

int main()
{
	float windmillAngle = 0.0f;
	
	pspDebugScreenInit();
	setupCallbacks();
	
	// *** Call this to init the graphics functions ***
	//
	initGraphics();
	
	// *** declare and load images. PNG files are supported ***
	//
	Image* backgroundImg;
	Image* spinningImg;
	Image* millImg;
	backgroundImg = loadImage("gfx/ground.png");
	spinningImg = loadImage("gfx/wind1.png");
	millImg = loadImage("gfx/wind2.png");
	if( backgroundImg == NULL || spinningImg == NULL || millImg==NULL)
	{
		sceKernelExitGame();
		return -1;
	}
	
	// *** scale the background to be twice its size
	// (this is necessary because the image dimensions are half
	// the screen width and height.)
	//
	stretchImage(backgroundImg,2.0f,2.0f);
	
	// *** we can set the Z location of the backgroundImg to -.9 to
	//	   make sure it is drawn at the bottom no matter what order
	//		the images are drawn. (All Z values must be a floating
	//		point number between -1 and 1.)
	// 	
	setImageZ(backgroundImg, -0.9f);
	
	setImageZ(millImg, -0.1f);
	
	// *** images can also be scaled down.
	//
	stretchImage(millImg,0.5f,0.5f);
	
	// *** this has to be called right before while(..)
	//
	sceKernelDcacheWritebackAll();
	
	
	// *** while the user hasn't exited with the home button...
	//
	while( running() )
	{
		// *** begin drawing sequence. All calls to drawImage go 
		//		between startDrawing() and endDrawing()
		//
		startDrawing();
		
		// *** render the background image at 0,0--the top left corner.
		// the last parameter is the angle in radians at which to draw.
		// specify 0 for no rotation, of course.
		//
		drawImage(backgroundImg,0,0,0.0f);
		
		// *** draw part of wind mill at a moving angle
		//
		drawImage(spinningImg,40,80,DEG2RAD(windmillAngle));
		
		// *** draw other part of windmill
		//
		drawImage(millImg,56,104,0.0f);
		
		// *** end the drawing sequence.
		//
		endDrawing();
		
		// *** flip the screen--this MUST be done AFTER endDrawing.
		//
		flipScreen();
		
		// *** printTextScreen calls must go after endDrawing()
		// and flipScreen(). It writes directly to vram.
		//
		char msg1[] = "Hazed Mirror Productions presents";
		char msg2[] = "Easy Accelerated Image lib 0.1";
		
		printTextScreen(msg1,100,130,0xff000000);
		printTextScreen(msg1,101,130,0xff000000);
		
		printTextScreen(msg2,100,138,0xff800000);
		printTextScreen(msg2,101,138,0xff800000);
		
		// *** avoid flickering by calling this after printing text
		//
		sceDisplayWaitVblankStart();
		
		windmillAngle += 0.2f; // rotate windmill
	}
	
	// *** call this to end the graphics functions lib***
	//
	endGraphics();
	
	// *** free images from memory
	//
	freeImage(backgroundImg);
	freeImage(spinningImg);
	freeImage(millImg);
	
	sceKernelExitGame();
	return 0;
}
